@@ -68,8 +68,15 @@ import {
   buildScaleKey,
   getStackingGroups,
   preparePlotData2,
+  AxisProps,
 } from '@grafana/ui/internal';
 
+import { ANNOTATION_LANE_SIZE } from '../../../plugins/panel/timeseries/plugins/utils';
+
+// See UPlotAxisBuilder.ts::calculateAxisSize for default axis size calculation
+export const UPLOT_DEFAULT_AXIS_SIZE = 17;
+export const UPLOT_DEFAULT_AXIS_GAP = 5;
+
 const defaultFormatter = (v: any, decimals: DecimalCount = 1) => (v == null ? '-' : v.toFixed(decimals));
 
 const defaultConfig: GraphFieldConfig = {
@@ -90,6 +97,7 @@ export const preparePlotConfigBuilder: UPlotConfigPrepFn = ({
   tweakAxis = (opts) => opts,
   hoverProximity,
   orientation = VizOrientation.Horizontal,
+  xAxisConfig,
 }) => {
   // we want the Auto and Horizontal orientation to default to Horizontal
   const isHorizontal = orientation !== VizOrientation.Vertical;
@@ -128,6 +136,10 @@ export const preparePlotConfigBuilder: UPlotConfigPrepFn = ({
       direction: isHorizontal ? ScaleDirection.Right : ScaleDirection.Up,
       isTime: true,
       range: () => {
+        const state = builder.getState();
+        if (state.isPanning) {
+          return [state.min, state.max];
+        }
         const r = getTimeRange();
         return [r.from.valueOf(), r.to.valueOf()];
       },
@@ -156,6 +168,10 @@ export const preparePlotConfigBuilder: UPlotConfigPrepFn = ({
         theme,
         grid: { show: i === 0 && xField.config.custom?.axisGridShow },
         filter: filterTicks,
+        formatValue: xField.config.unit?.startsWith('time:')
+          ? (v, decimals) => xField.display!(v, decimals).text
+          : undefined,
+        ...xAxisConfig,
       });
     }
 
@@ -706,3 +722,25 @@ function getNamesToFieldIndex(frame: DataFrame, allFrames: DataFrame[]): Map<str
   });
   return originNames;
 }
+
+export function getXAxisConfig(lanes = 1): Pick<AxisProps, 'size' | 'gap' | 'ticks'> | undefined {
+  if (lanes > 1) {
+    const annotationLanesSize = lanes * ANNOTATION_LANE_SIZE;
+    // Add an extra lane's worth of height below the annotation lanes in order to show the gridlines through the annotation lanes
+    const axisSize = annotationLanesSize + UPLOT_DEFAULT_AXIS_GAP;
+    // Consistent gap between gridlines and x-axis labels
+    const gap = UPLOT_DEFAULT_AXIS_GAP;
+    // Axis size is: default size + gap size + annotationLaneSize
+    const size = UPLOT_DEFAULT_AXIS_SIZE + gap + annotationLanesSize;
+
+    return {
+      size,
+      gap,
+      ticks: {
+        size: axisSize,
+      },
+    };
+  }
+
+  return undefined;
+}
